const Logger = console;
let Vue;
let GLOBAL_THERSHOLD;
let observer;
let count = 0;
const unit = 0.1;
let threshold = [];
while (count <= 1) {
    threshold.push(count);
    count = Number((count + unit).toFixed(2));
}
const OBSERVER_OPTIONS = {
    delay: 100,
    threshold: threshold,
    trackVisibility: true,
};
const elToMeta = new Map();
const isObjectHandler = (value) => {
    if (!value || typeof value !== 'object') {
        return false;
    }
    if (!value.enter && !value.leave) {
        return false;
    }
    if (value.enter && typeof value.enter !== 'function') {
        return false;
    }
    if (value.leave && typeof value.leave !== 'function') {
        return false;
    }
    return true;
};
const isFuncHandler = (value) => {
    if (typeof value === 'function') {
        return true;
    }
    return false;
};
const isExposureHandler = (value) => {
    return isObjectHandler(value) || isFuncHandler(value);
};
const isVisibleElement = (el) => {
    const { visibility, height, width } = window.getComputedStyle(el, null);
    if (visibility === 'hidden' ||
        parseInt(height) === 0 ||
        parseInt(width) === 0) {
        return false;
    }
    return true;
};
const useResetExposure = function (el) {
    if (el && elToMeta.has(el)) {
        const config = elToMeta.get(el);
        elToMeta.set(el, Object.assign(config, {
            active: {
                enter: false,
                leave: false,
            },
        }));
    }
    else {
        for (let [key, config] of elToMeta.entries()) {
            elToMeta.set(key, Object.assign(config, { active: { enter: false, leave: false } }));
        }
    }
};
const createObserver = () => {
    if (window.IntersectionObserver && !observer) {
        observer = new window.IntersectionObserver((list, observer) => {
            for (let entry of list) {
                const { isIntersecting, target, intersectionRatio } = entry;
                const config = elToMeta.get(target);
                if (config) {
                    if (isFuncHandler(config.handler) && config.active.enter) {
                        break;
                    }
                    if (isObjectHandler(config.handler) &&
                        config.active.enter &&
                        config.active.leave) {
                        break;
                    }
                    if (!isVisibleElement(target)) {
                        break;
                    }
                    if (isIntersecting &&
                        !config.active.enter &&
                        intersectionRatio >= config.threshold) {
                        if (isFuncHandler(config.handler)) {
                            config.handler(target);
                        }
                        else {
                            config.handler.enter && config.handler.enter(target);
                        }
                        config.active.enter = true;
                    }
                    else {
                        if (isObjectHandler(config.handler) &&
                            !config.active.leave &&
                            config.active.enter &&
                            intersectionRatio <= 0) {
                            config.handler.leave && config.handler.leave(target);
                            config.active.leave = true;
                        }
                    }
                    elToMeta.set(target, config);
                }
            }
        }, OBSERVER_OPTIONS);
    }
    if (!observer) {
        Logger.warn('current browser does not support IntersectionObserve API');
    }
};
const addElToObserve = (el, threshold, handler, context) => {
    if (!elToMeta.has(el)) {
        elToMeta.set(el, {
            active: {
                enter: false,
                leave: false,
            },
            handler,
            context,
            threshold,
        });
        observer && observer.observe(el);
    }
};
const createExposureObserver = (option) => {
    const threshold = option.threshold || GLOBAL_THERSHOLD;
    return function observe(el, handler) {
        if (!isExposureHandler(handler)) {
            Logger.error(`handler is not ExposureHandler. 
         ExposureHandler type:
          - function: (el?: Element) => void
          - object: {enter?: (el?: Element) => void, leave?: (el?: Element) => void}
         `);
            return;
        }
        if (!elToMeta.has(el)) {
            elToMeta.set(el, {
                active: {
                    enter: false,
                    leave: false,
                },
                context: option.context,
                handler,
                threshold,
            });
            observer && observer.observe(el);
        }
    };
};
const bind = (el, binding, vnode) => {
    let { value, arg } = binding;
    let threshold;
    const { context } = vnode;
    if (!isExposureHandler(value)) {
        Logger.error(`directive value is not ExposureHandler. 
       ExposureHandler type:
        - function: (el?: Element) => void
        - object: {enter?: (el?: Element) => void, leave?: (el?: Element) => void}
       `);
        return;
    }
    if (!context) {
        return;
    }
    threshold = Number(arg);
    if ((arg && typeof arg !== 'number') || !arg) {
        arg && Logger.error('element arguments  must be number type');
        threshold = GLOBAL_THERSHOLD || 1;
    }
    if (context.$resetExposure && context.$resetExposure !== useResetExposure) {
        Logger.error('context bind $resetExposure propertyKey');
        return;
    }
    !context.$resetExposure && (context.$resetExposure = useResetExposure);
    addElToObserve(el, threshold, value, context);
};
const unbind = (el) => {
    if (elToMeta.has(el) && observer) {
        elToMeta.delete(el);
        observer.unobserve(el);
    }
};
const installDirective = () => {
    Vue.directive('exposure', {
        bind,
        unbind,
    });
};
const install = (_Vue, options) => {
    if (!Vue) {
        Vue = _Vue;
    }
    if (options && options.threshold) {
        GLOBAL_THERSHOLD = options.threshold;
    }
    createObserver();
    installDirective();
};
const Exposure = {
    install,
};

export { createExposureObserver, Exposure as default, useResetExposure };
